/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//------------------------------------------------------------------------------
//
// GLUT/OpenGL-based catenary curve drawing program
//
//
//  Created: J. Tumblin 12/16/04
//
===============================================================================*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>

#define TITLE "Catenary Curve Construction"
#define W_WIDTH		704			// on-screen display window width, height
#define W_HEIGHT	704

/* Near and far planes of the frustum */
#define NEAR 0.1
#define FAR 10

#define M_PI	3.1415926535897932384626433832795
#define BOOL	short int
#define TRUE	1
#define FALSE	0

#define JT_XORIG	W_WIDTH/3				// on-screen origin for the drawing.
#define JT_YORIG	W_HEIGHT/3
#define JT_XSCALE	W_WIDTH/4 
#define JT_YSCALE	W_HEIGHT/4

// Function prototypes:		// (these should go in a .h file if it exists.)
//==============================================================================
void display();
void reshape(int w, int h);
void gl_init();
void glut_init(int *argc, char **argv);
void keyboard(unsigned char key, int x, int y);
void keyspecial(int key, int x, int y);				// respond to arrow keys.
void drawDot(double xp, double yp, double r, double vcnt=10);

class Catenary
//==============================================================================
// A bilinearly-ruled quadrilateral.  'Pt' array holds 4 vertices in counter-
// clockwise order, given in x,y coordinates.  We define a 'u,v' coordinate
// system with its orgin at point p0,
{
public:
	double pt[4][2];			// Coordinates for vertices. pt[n][c] describes
								// vertex # n, and coordinate c. For example,
								// Point P2 is (x,y) == pt[2][0], pt[2][1]);
	int urule,vrule;			// # of grid lines in u,v directions.
	int selectPt;				// point selected for user input/update

public:
	Catenary::Catenary(void);		// default constructor
	Catenary::~Catenary(void);		// default destructor
	void zero(void);					// set all points to zero.
	void unitSquare(void);				// set points to unit-square shape
	void setRule(int ucnt,int vcnt);	// set # of lines in u,v.
	void drawMe(void);				// Use OpenGL calls to draw the catenary.
	keyspecial(int key, int x, int y);	// respond to arrow keys.
};

// YIKES!!! Global variables. How do you avoid them in GLUT?
//==============================================================================
Catenary cat1;					// the catenary curve object we're making.


int main(int argc, char** argv)
//==============================================================================
{
	glut_init(&argc, argv);				// First initialize GLUT,
	gl_init();							// then initialize OpenGL stuff,
	cat1.setRule(5,5);					// draw 5 lines in u,v
	glutMainLoop(); 					// finally, the GLUT infinite loop.

	return 0;
}
void gl_init()
//------------------------------------------------------------------------------
// All OpenGL related initializing
{
	glClearColor(0.0, 0.0, 0.0, 0.0);	// Set color for screen-clearing.

	glColor3f(0.0, 0.0, 0.0);			// Select current color 
	glShadeModel(GL_FLAT);				// Flat shading model 
	glDisable(GL_LIGHTING);				// No lighting needed
	glMatrixMode(GL_MODELVIEW);			// use identity matrix
	glLoadIdentity();					// for the modelview transform.

											//Clear the Z- and frame-buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

									// Set camera viewing params
	gluLookAt(0.0, 0.0,  1.0,	    // eyepoint at 1 on X axis, 
			  0.0, 0.0, -1.0,		// look down the negative Z-axis
			  0.0, 1.0, 0.0);		// Y axis is the 'up' direction .

	
	
	cat1.unitSquare();					// Initially we make a unit-square.

}

void glut_init(int *argc, char **argv)
//------------------------------------------------------------------------------
// All GLUT-related initializing.
{
	glutInit(argc, argv);				// GLUT master initializing fcn.

						// single buffer, RGB color model, depth buffer 
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);

						// Display window parameters, creation.
	glutInitWindowSize(W_WIDTH, W_HEIGHT);
	glutInitWindowPosition(0, 0);
	glutCreateWindow(TITLE);

	//...................................... Register GLUT Callback functions
	glutDisplayFunc(display);			// call this fcn for display,
	glutKeyboardFunc(keyboard);			// call this one for keyboard,
	glutSpecialFunc(keyspecial);		// call this one for arrow & other keys
	glutReshapeFunc(reshape);			// call this one for reshape, etc.
//	glutIdleFunc(idle);
}

void display()
//------------------------------------------------------------------------------
// GLUT calls this display function to redraw the screen; your code can force
// redraw with a call to glutPostRedisplay(). Put all drawing code here.
//  (See 'Reshape' for viewing transforms)
{
	glClear(GL_COLOR_BUFFER_BIT);
	cat1.drawMe();					// Draw the catenary curve.
    glFlush();						// Execute all pending drawing commands.
}

void reshape(int w, int h)
//------------------------------------------------------------------------------
//glut reshape function. Called when the window is resized. Also called when 
// the window is created. Initialize the Projection matrix here.
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);

	/* Set the progection matrix */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// multiply the current viewing transform (identity) by orthographic camera
	glOrtho(0.0, W_WIDTH,	// left,right	
		    0.0, W_HEIGHT,	// bottom,top
			1.0,-1.0);	// near, far clipping planes.

	/* Initialize the modelview matrix */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void keyboard(unsigned char key, int x, int y)
//------------------------------------------------------------------------------
// Called by GLUT when user hits a key
{
	printf("key %d\n",(int)key);
	switch(key) {
		case 27: /* Esc */
		case 'Q':
		case 'q':
			exit(0); /* Quit from application */
			break;
		case '0':								// select p0 vertex to move.
			cat1.selectPt = 0;
			break;
		case '1':
			cat1.selectPt = 1;
			break;
		case '2':
			cat1.selectPt = 2;
			break;
		case '3':
			cat1.selectPt = 3;
			break;
		default:
			printf("?!?! I don't know that key\n");
			glutPostRedisplay();
			break;
	}
}

void keyspecial(int key, int x, int y)
//------------------------------------------------------------------------------
// Called when user presses special, non-alphanumeric keys.
{
int isCorner;

	// make sure we have selected one of the control points.
	if(cat1.selectPt > 3 || cat1.selectPt < 0) cat1.selectPt =0;
	isCorner = 1;					// assume user adjusted a corner position
	switch(key)
	{
	case GLUT_KEY_RIGHT:							// move the selected point
		cat1.pt[cat1.selectPt][0] += 0.1;
		break;
	case GLUT_KEY_LEFT:
		cat1.pt[cat1.selectPt][0] -= 0.1;
		break;
	case GLUT_KEY_UP:
		cat1.pt[cat1.selectPt][1] += 0.1;
		break;
	case GLUT_KEY_DOWN:
		cat1.pt[cat1.selectPt][1] -= 0.1;
		break;
	case GLUT_KEY_PAGE_UP:							// adjust # of ruled lines
		cat1.setRule(cat1.urule+1,cat1.vrule+1);
		isCorner = 0;								// (corners are unchanged)
		break;
	case GLUT_KEY_PAGE_DOWN:								
		cat1.setRule(cat1.urule-1,cat1.vrule-1);
		if(cat1.urule<2) cat1.urule=2;				// 2 ruled lines: always
		if(cat1.vrule<2) cat1.vrule=2;				// draw the quadrilateral.
		isCorner = 0;								// (corners are unchanged)
		break;
	default:
		return;		// without glutPostRedisplay().
		break;
	}
	glutPostRedisplay();		// Something changed; redraw the screen.
	if(isCorner==1)				// if user changed corner position,
	{							// report the new ones.
		printf("\n P3: %f, %f \t P2: %f, %f \n P0: %f, %f \t P1: %f, %f\n",
			cat1.pt[3][0],cat1.pt[3][1],	cat1.pt[2][0],cat1.pt[2][1],
			cat1.pt[0][0],cat1.pt[0][1],	cat1.pt[1][0],cat1.pt[1][1]);
	}
	else
	{							// if user changed # of ruled lines, report it.
		printf(" u,v subdivided into %d pieces\n",cat1.urule-1);
	}
}

void drawDot(double xp, double yp, double r, double vcnt)
//------------------------------------------------------------------------------
// Using 'vcnt' vertices, draw a filled circle of radius 'r' centered at xp,yp.
{
int i;
double dtheta,x,y;

	dtheta = 2.0 * M_PI / vcnt;		// step size.
	glBegin(GL_POLYGON);
	for(i=0; i< vcnt; i++)
	{
		x = JT_XORIG + xp*JT_XSCALE + r*cos(dtheta*i);
		y = JT_YORIG + yp*JT_YSCALE + r*sin(dtheta*i);
		glVertex2f(x,y);
	}
	glEnd();

}

Catenary::Catenary(void)
//------------------------------------------------------------------------------
// default constructor
{
	zero();
	setRule(2,2);					// draw only the outline.
	selectPt = 0;					// assume 1st user-adjusted pt is p0.
}

Catenary::~Catenary(void)
//------------------------------------------------------------------------------
// default destructor
{
}

void Catenary::zero(void)
//------------------------------------------------------------------------------
// set all points to zero.
{
int i,j;

	for(i=0; i<4; i++)
	{
		for(j=0; j<2; j++)
		{
			pt[i][j] = 0.0;
		}
	}
}

void Catenary::unitSquare(void)
//------------------------------------------------------------------------------
// set points to unit-square shape
{
	pt[0][0] = 0.0; pt[0][1] = 0.0;			// P0
	pt[1][0] = 1.0; pt[1][1] = 0.0;			// P1
	pt[2][0] = 1.0; pt[2][1] = 1.0;			// P2
	pt[3][0] = 0.0; pt[3][1] = 1.0;			// P3
}

void Catenary::setRule(int ucnt, int vcnt)
//------------------------------------------------------------------------------
// Set number of rules lines in u, v directions.
{
	urule = ucnt;
	vrule = vcnt;
}

void Catenary::drawMe(void)
//------------------------------------------------------------------------------
// Use OpenGL calls to draw the catenary.
{
int i;							// line-number.
double frac;					
double xbgn, ybgn;				// line beginning point,
double xend, yend;				// line ending point.

	glColor3f(1.0,0.7,0.0);					// Set Vertex color
	for(i=0; i< 4; i++)						// draw vertices
	{
		drawDot(pt[i][0], pt[i][1], 3.0);		
	}
	// Draw bilinear catenary lines. 
	//  For lines of constant 'u', interpolate between p0 and p1 for starting pt
	//		and between p3 and p2 for ending point.
	//	For lines of constant 'v', interpolate between p0 and p3 for starting pt
	//		and between p1 and p2 for ending point.
	
	frac = 1.0 / (urule-1);		// spacing between endpoints, measured in 'u'
	glBegin(GL_LINES);
	for(i=0; i<urule; i++)				// divide 'u' into 'urule' lines:
	{
		if(i==0 || i==vrule-1) glColor3f(1.0,1.0,1.0);	//white outline color
		else glColor3f(0.0, 1.0, 0.0);	// green'u' line color
		xbgn = pt[0][0] + i*frac*(pt[1][0] - pt[0][0]);
		ybgn = pt[0][1] + i*frac*(pt[1][1] - pt[0][1]);
		glVertex2d(JT_XORIG + xbgn*JT_XSCALE,JT_YORIG + ybgn*JT_YSCALE);

		xend = pt[3][0] + i*frac*(pt[2][0] - pt[3][0]);
		yend = pt[3][1] + i*frac*(pt[2][1] - pt[3][1]);
		glVertex2d(JT_XORIG + xend*JT_XSCALE,JT_YORIG + yend*JT_YSCALE);
	}	
	glEnd();
	frac = 1.0 / (vrule-1);		// spacing between endpoints, measured in 'u'

	glBegin(GL_LINES);
	for(i=0; i<vrule; i++)				// divide 'v' into 'vrule' lines:
	{
		if(i==0 || i==vrule-1) glColor3f(1.0,1.0,1.0);	//white outline color
		else glColor3f(1.0, 0.0, 1.0);			// purple 'v' line color
		xbgn = pt[0][0] + i*frac*(pt[3][0] - pt[0][0]);
		ybgn = pt[0][1] + i*frac*(pt[3][1] - pt[0][1]);
		glVertex2d(JT_XORIG + xbgn*JT_XSCALE,JT_YORIG + ybgn*JT_YSCALE);

		xend = pt[1][0] + i*frac*(pt[2][0] - pt[1][0]);
		yend = pt[1][1] + i*frac*(pt[2][1] - pt[1][1]);
		glVertex2d(JT_XORIG + xend*JT_XSCALE,JT_YORIG + yend*JT_YSCALE);

	}	
	glEnd();
}

